import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { Resource } from 'src/app/models/resource.model';
import { ResourceService } from 'src/app/services/resource.service';
import { FormBuilder, FormGroup, Validators, FormArray } from '@angular/forms';

@Component({
  selector: 'app-resource-form',
  templateUrl: './resource-form.component.html',
  styleUrls: ['./resource-form.component.scss'],
})
export class ResourceFormComponent implements OnInit {

  resource: Resource;
  form: FormGroup;
  title: string;

  constructor(
    private resourceService: ResourceService,
    private route: ActivatedRoute,
    private router: Router,
    private _fb: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.title = this.route.snapshot.data.title;
    this.initForm();

    this.route.paramMap
      .filter((paramMap: ParamMap) => (paramMap.get('id') !== null))
      .switchMap((paramMap: ParamMap) => this.resourceService.findById(paramMap.get('id')))
      .subscribe((resource: Resource) => {

        this.resource = resource;

        this.form = this._fb.group(
          {
            id: [this.resource.id],
            name: [resource.name, Validators.required],
            type: [resource.type, Validators.required],
            queryable: [resource.queryable, Validators.required],
            downloadable: [this.resource.downloadable, Validators.required],
            description: [resource.description],
            outputFormats: [this.resource.outputFormats.join(',')],
          });

      });

  }

  initForm() {
    this.resource = new Resource();
    const arr = new FormArray([]);
    this.form = this._fb.group(
      {
        id: [this.resource.id],
        name: [this.resource.name, Validators.required],
        type: [this.resource.type, Validators.required],
        queryable: [this.resource.queryable, Validators.required],
        downloadable: [this.resource.downloadable, Validators.required],
        description: [this.resource.description],
        outputFormats: [this.resource.outputFormats.join(',')],
      });
  }

  onSubmit() {
    if (!this.formInvalid) {
      this.resource = new Resource(this.form.value);
      this.resource.outputFormats = this.form.value.outputFormats.split(',');
      this.resourceService.replaceOrCreate(this.resource)
        .subscribe(
          (resourceCreated) => {
            this.router.navigate(['/resources', resourceCreated.id])
              .then(() => {
              });
          },
          (err) => {
            alert(err.message);
          },
        );
    }
  }

  // Getters for each property
  get name() {
    return this.form.controls['name'];
  }

  get type() {
    return this.form.controls['type'];
  }

  get queryable() {
    return this.form.controls['queryable'];
  }

  get description() {
    return this.form.controls['description'];
  }

  get outputFormats() {
    return this.form.controls['outputFormats'];
  }

  get formInvalid() {
    return this.form.invalid;
  }
}
